# beetbase shiny app
if(!require(shiny)) install.packages("shiny", repos = "http://cran.us.r-project.org")
if(!require(shinydashboard)) install.packages("shinydashboard", repos = "http://cran.us.r-project.org")
if(!require(bslib)) install.packages("bslib", repos = "http://cran.us.r-project.org")
if(!require(shinyjs)) install.packages("shinyjs", repos = "http://cran.us.r-project.org")
if(!require(rintrojs)) install.packages("rintrojs", repos = "http://cran.us.r-project.org")
if(!require(ggplot2)) install.packages("ggplot2", repos = "http://cran.us.r-project.org")
if(!require(ggiraph)) install.packages("ggiraph", repos = "http://cran.us.r-project.org")
if(!require(tidyr)) install.packages("tidyr", repos = "http://cran.us.r-project.org")
if(!require(dplyr)) install.packages("dplyr", repos = "http://cran.us.r-project.org")
if(!require(cowplot)) install.packages("cowplot", repos = "http://cran.us.r-project.org")
if(!require(htmlwidgets)) install.packages("htmlwidgets", repos = "http://cran.us.r-project.org")
if(!require(leaflet)) install.packages("leaflet", repos = "http://cran.us.r-project.org")
if(!require(grid)) install.packages("grid", repos = "http://cran.us.r-project.org")
if(!require(DT)) install.packages("DT", repos = "http://cran.us.r-project.org")

########## set color for each species ##########
species_color = c('Beta_vulgaris_L_subsp_vulgaris'='#800000', 
  'Beta_vulgaris_L_subsp_vulgaris'='#800000', 
  'Beta_vulgaris_L'='#800000',
  'Beta_vulgaris_L_subsp_maritima'='#9A6324',
  'Beta_macrocarpa'="#808000",
  'Beta_trigyna'='#469990',
  'Beta_nana'='#000075',
  'Beta_hybr'='#e6194B',
  'Beta_patula'='#f58231',
  'Beta_lomatogona'='#ffe119',
  'Beta_macrorhiza'='#bfef45',
  'Beta_corolliflora'='#3cb44b',
  'Patellifolia_patellaris'='#42d4f4',
  'Patellifolia_procumbens'='#4363d8',
  'Beta_intermedia'='#911eb4',
  'Patellifolia_webbiana'='#f032e6')

############## GRIN dataset ###############
#/home/kevin-dorn/shiny_phenomics
GRIN_data <- read.csv(file = 'data/DescriptorsGRIN_Global.csv', sep = ",", header = TRUE)
drops <- c("X", "AVAILABILITY", "ID")
GRIN_data_cleaned <- dplyr::select(GRIN_data, -c("X", "AVAILABILITY", "ID"))
GRIN_data_cleaned <- GRIN_data_cleaned %>% separate(CERCOSPORA, c('CERCOSPORA', 'CERCOSPORA_scale'), sep = ":")
GRIN_data_cleaned <- GRIN_data_cleaned %>% separate(CURLY.TOP, c('CURLY_TOP', 'CURLY_TOP_scale'), sep = ":")
GRIN_data_cleaned <- GRIN_data_cleaned %>% separate(RHIZOCTONIA, c('RHIZOCTONIA', 'RHIZOCTONIA_scale'), sep = ":")
GRIN_data_cleaned <- GRIN_data_cleaned %>% separate(RHIZOMANIA, c('RHIZOMANIA', 'RHIZOMANIA_scale'), sep = ":")


############## RCRR DI over time ###############
DI_over_time <- read.csv(file = 'data/DI_over_time.csv', sep = ",", header = TRUE)
colnames(DI_over_time) = c("Year", "Experiment", "Type", "Company", "Source", "Rep", 
                           "plants", "0's", "1's", "2's", "3's", "4's", "5's", "6's", 
                           "7's", "DI", "Healthy", "Harvestable")
DI_over_time_short <- select(DI_over_time, -Healthy, -Harvestable)
DI_over_time_short$DI <- as.numeric(DI_over_time_short$DI) 


########## Genome data ############
EL10_chr_names <- c("EL10_2_Chr1","EL10_2_Chr2","EL10_2_Chr3","EL10_2_Chr3_s1","EL10_2_Chr4",
                    "EL10_2_Chr5","EL10_2_Chr5_s1","EL10_2_Chr5_s2","EL10_2_Chr6","EL10_2_Chr7",
                    "EL10_2_Chr8","EL10_2_Chr9","EL10_2_Chr_UnSca1","EL10_2_Chr_UnSca2",
                    "EL10_2_Chr_UnSca3","EL10_2_Chr_UnSca4","EL10_2_Chr_UnSca5","EL10_2_Chr_UnSca6")


########## Define Shiny UI for application that draws a histogram ###########

ui <- dashboardPage(
  skin = "green",
  #title = "BeetBase",
  header = dashboardHeader(titleWidth = '100%',
			   disable = TRUE,
                           title = "BeetBase: A Sugar Beet Toolbox for Genetics, Genomics and Germplasm Research"
                           ),
  
  dashboardSidebar(sidebarMenu(id = "menu", sidebarMenuOutput("menu")), 
                   width = 200, collapsed = FALSE, disable = TRUE
                   #tags$script(JS("document.getElementsByClassName('sidebar-toggle')[0].style.visibility = 'hidden';"))
                   ), 
  
  ############################
  dashboardBody(
    tags$style('.content-wrapper  {
                 background-color: #FFFFFF;
      }'),
    fluidPage(
            h1("Rhizoctonia Root and Crown Rot resistance for geo-referenced Beta accessions"), 
            fluidRow(
            column(7,h4("This tool displays data from the annual sugar beet crop germplasm committee Rhizoctonia 
                nursery in Fort Collins, Colorado. Data for all screened and geo-referenced accessions
                is included, and will be updated annually as more accessions are screened."))),
  
            h4("How to use this tool:", style = "font-weight:bold"),
            h4("Select one or multiple species of interest and identify
               either desired country of origin or Rhizoctonia rating to
               display trends on the interactive map"),
            hr(),
            sidebarLayout(
            # Sidebar panel for inputs ----
              sidebarPanel(
              # Copy the line below to make a slider bar 
              selectInput("species_name", label = h3("Species (Select one or multiple)"),
                       c("Beta vulgaris", 
                         "Beta vulgaris spp. vulgaris", 
                         "Beta vulgaris spp. maritima", 
                         "Beta nana",
                         "Beta trigyna",
                         "Beta macrocarpa",
                         "Beta macrorhiza",
                         "Beta corolliflora",
                         "Patellifolia procumbens",
                         "Patellifolia webbiana",
                         "Patellifolia patellaris"),
                       multiple = TRUE, selected = c("Beta vulgaris spp. vulgaris", 
                                                     "Beta vulgaris spp. maritima")),
           
              # Copy the line below to make a slider bar 
              selectInput("country", label = h3("Country of Origin"), 
                       c("Albania",
                         "Armenia",
                         "Belgium",
                         "Bulgaria",
                         "Croatia",
                         "Cyprus",
                         "Denmark",
                         "France",
                         "Georgia",
                         "Germany",
                         "Greece",
                         "Ireland",
                         "Italy",
                         "Kazakhstan",
                         "Mongolia",
                         "Morocco",
                         "Netherlands",
                         "Poland",
                         "Portugal",
                         "Russian Federation",
                         "Spain",
                         "Sri Lanka",
                         "Syria",
                         "Tajikistan",
                         "United Kingdom"),
                       multiple = TRUE, selected = c("France",
                                                     "Germany",
                                                     "Italy",
                                                     "Spain")),
           
           
               sliderInput("Rhizoctonia", label = h3("Rhizoctonia Rating (0:healthy, 7:dead)"), min = 0, 
                           max = 7, value = 7),
               hr(),
               downloadButton("downloadData", "Download")
           
                ),
         
              mainPanel(
              column(7, leafletOutput("map", height = 600, width = 1000))
              )
            )
        
      
    )
  )
)


# -----------------------------------------------------------------------
# -------------------------------SERVER ---------------------------------
# -----------------------------------------------------------------------

# Define server logic required to draw a histogram
server <- function(input, output, session) {
  ############# make sidebarmenu ###########
  output$menu <- renderMenu({
    sidebarMenu(
      menuItem("Dashboard", tabName = "dashboard", icon = NULL),
      menuItem("Disease Screening", icon = NULL,
              menuSubItem("Rhizoctonia", tabName = "rhizoctonia", icon = NULL),
              menuSubItem("Curly Top", tabName = "curly_top", icon = NULL)
              ),
      menuItem("Genomics", tabName = "genomics", icon = NULL,
              menuSubItem("Genome Browser", tabName = "jbrowse", icon = NULL),
              menuSubItem("FTP Download", tabName = "ftpdownload", icon = NULL),
              menuSubItem("Genetic Variant Discovery", tabName = "variantdiscovery", icon = NULL),
              menuSubItem("Protein Models", tabName = "proteinmodels", icon = NULL)
              ),
      menuItem("GRIN data", tabName = "GRINdata", icon = NULL)
      )
  })
  

  ######### Add GRIN data #########
  all_GRIN_data <- read.csv(file = 'data/lat_long_GRIN_v1GRIN_out.csv', 
                            sep = ",", header = TRUE)
  GRIN_data <- read.csv(file = 'data/lat_long_only.csv', sep = "\t", header = TRUE)
  alt_names <- read.csv(file = 'data/alt_name_data.csv', sep = "\t", header = TRUE)
  
  circle_color_pal <- colorFactor(c('#e6194b', '#3cb44b', '#ffe119', '#4363d8', 
                                    '#f58231', '#911eb4', '#46f0f0', '#f032e6', 
                                    '#bcf60c', '#fabebe', '#008080'),
                                  domain = c("Beta vulgaris", 
                                             "Beta vulgaris subsp. vulgaris", 
                                             "Beta vulgaris subsp. maritima", 
                                             "Beta nana",
                                             "Beta trigyna",
                                             "Beta macrocarpa",
                                             "Beta macrorhiza",
                                             "Beta corolliflora",
                                             "Patellifolia procumbens",
                                             "Patellifolia webbiana",
                                             "Patellifolia patellaris"))
  
  data_filtered = GRIN_data
  
  ########### filter data ##########
  filtered_data <- reactive({
    rhizoc_score_filter <- input$Rhizoctonia
    #rhizoc_score_filter_min <- input$Rhizoctonia[1]
    #cercospora_score_filter_min <- input$Cercospora[1]
    #rhizoc_score_filter_max <- input$Rhizoctonia[2]
    #cercospora_score_filter_max <- input$Cercospora[2]
    
    if(is.null(input$species_name)){
      species_filter_value <- list("Beta vulgaris spp. vulgaris", 
                                   "Beta vulgaris spp. maritima" 
                                   )
    }
    else{
      species_filter_value <- input$species_name
    }
    
    if(is.null(input$country)){
      country_filter_value <- list("France",
                                   "Germany",
                                   "Italy",
                                   "Spain"
                                   )
    }
    else{
      country_filter_value <- input$country
    }
    
    filtered_lat_long_GRIN_DATA <- GRIN_data %>% 
      dplyr::filter(TAXONOMY %in% species_filter_value)%>% 
      dplyr::filter(ORIGIN %in% country_filter_value)%>%
      dplyr::filter(RHIZOCTONIA <= rhizoc_score_filter)#%>%
    #dplyr::filter(CERCOSPORA <= cercospora_score_filter_max)%>%
    filtered_lat_long_GRIN_DATA
  })
  
  
  ########### render leafletplot ##########
  output$map <- renderLeaflet({
    
    if (!is.null(filtered_data())){
      data_filtered = filtered_data()
    }
    
    map_plot <- leaflet(data_filtered) %>% 
      setView(lng = 28, lat = 43, zoom = 3) %>%
      addProviderTiles("OpenStreetMap", group="OpenStreetMap") %>%
      addProviderTiles("CartoDB.Positron",group = "CartoDB.Positron") %>% 
      addProviderTiles("Esri.WorldImagery",group = "Esri.WorldImagery") %>% 
      addLayersControl(baseGroups = c("CartoDB.Positron",
                                      "OpenStreetMap", 
                                      "Esri.WorldImagery"), 
                       position = "topright") %>%
      addCircles(lat = ~data_filtered$latitude,
                 lng = ~data_filtered$longitude,
                 color = "red",
                 radius = sqrt(18^(11-data_filtered$RHIZOCTONIA)),
                 popup = paste("<strong>GRIN ID: </strong>", data_filtered$ID, 
                               "<br><strong>Country: </strong>", data_filtered$ORIGIN, 
                               "<br><strong>Rhizoctonia score: </strong>", data_filtered$RHIZOCTONIA, 
                               "<br><strong>PI: </strong>", data_filtered$ACCESSION))
    
    map_plot
    })
  output$downloadData <- downloadHandler(
    filename = function() {
      paste("germplasm_search", ".csv", sep = "")
    },
    content = function(file) {
      write.csv(filtered_data(), file, row.names = FALSE)
    }
  )
}

# Run the application 
shinyApp(ui, server, options = list(
  port = 7229)
  )
